@@ -7,8 +7,8 @@ struct ops o;  /* option structure */
 
 int main(int argc, char *argv[]) {
 int i, j, arg, argvlen;
-short fastscan=0, tcpscan=0, udpscan=0, randomize=0, resolve_all=0;
-short quashargv = 0, pingscan = 0, lamerscan = 0;
+short fastscan=0, tcpscan=0, randomize=1, resolve_all=0;
+short quashargv = 0, pingscan = 0;
 int lookahead = LOOKAHEAD;
 short bouncescan = 0;
 unsigned short *ports = NULL;
@@ -38,10 +38,15 @@ options_init();
 
 emptystring[0] = '\0'; /* It wouldn't be an emptystring w/o this ;) */
 
+/* Trap these sigs for cleanup */
+signal(SIGINT, sigdie);
+signal(SIGTERM, sigdie);
+signal(SIGHUP, sigdie); 
+
 if (argc < 2 ) printusage(argv[0]);
 
 /* OK, lets parse these args! */
-while((arg = getopt(argc,fakeargv,"Ab:DdFfhiL:lM:NnPp:qrRS:sT:tUuw:Vv")) != EOF) {
+while((arg = getopt(argc,fakeargv,"Ab:DdFfhiL:lM:Nno:Pp:qrRS:sT:tUuw:Vv")) != EOF) {
   switch(arg) {
   case 'A': o.allowall++; break;
   case 'b': 
@@ -58,7 +63,7 @@ while((arg = getopt(argc,fakeargv,"Ab:DdFfhiL:lM:NnPp:qrRS:sT:tUuw:Vv")) != EOF)
   case '?': printusage(argv[0]);
   case 'i': o.identscan++; break;
   case 'L': lookahead = atoi(optarg); break;
-  case 'l': lamerscan++; udpscan++; break;
+  case 'l': o.lamerscan++; o.udpscan++; break;
   case 'M': 
     o.max_sockets = atoi(optarg); 
     if (o.max_sockets > MAX_SOCKETS_ALLOWED) {
@@ -68,25 +73,35 @@ while((arg = getopt(argc,fakeargv,"Ab:DdFfhiL:lM:NnPp:qrRS:sT:tUuw:Vv")) != EOF)
     break;
   case 'n': o.noresolve++; break;
   case 'N': o.force++; break;
+  case 'o': 
+    if (o.logfd) fatal("Only one log filename allowed");
+    o.logfd = fopen(optarg, "w");
+    if (!o.logfd) 
+      fatal("Failed to open output file %s for writing", optarg);
+    break;
   case 'P': pingscan++; break;
   case 'p': 
     if (ports)
       fatal("Only 1 -p option allowed, seperate multiple ranges with commas.");
     ports = getpts(optarg); break;
   case 'R': resolve_all++; break;
-  case 'r': randomize++; break;
+  case 'r': 
+    randomize = 0;
+    printf("Warning: Randomize syntax has been changed, -r now requests that ports NOT be randomized\n");
+    break;
   case 's': o.synscan++; break;
   case 'S': 
-    if (source)
+    if (o.spoofsource)
       fatal("You can only use the source option once!\n");
     source = safe_malloc(sizeof(struct in_addr));
+    o.spoofsource = 1;
     if (!resolve(optarg, source))
       fatal("Failed to resolve source address, try dotted decimal of check for type\n");
     break;
   case 'T': o.ptime = atoi(optarg); break;
   case 't': tcpscan++; break;
   case 'U': o.finscan++; break;
-  case 'u': udpscan++; break;
+  case 'u': o.udpscan++; break;
   case 'q': quashargv++; break;
   case 'w': o.wait = atoi(optarg); break;
   case 'V': 
@@ -103,16 +118,21 @@ if (!o.isr00t && pingscan) fatal("You can't do a ping scan if you aren't root");
 if (pingscan && o.dontping) fatal("ICMP ping scan -P and don't ping -D are incompatible options, Duh.");
 if (!o.isr00t) o.dontping++;
 if (bouncescan && !o.dontping) printf("Hint: if your bounce scan target hosts aren't reachable from here, remember to use -D\n");
-if (tcpscan && o.synscan) 
-  fatal("The -t and -s options can't be used together.\
- If you are trying to do TCP SYN scanning, just use -s.\
- For normal connect() style scanning, use -t");
+if (tcpscan && (o.synscan || o.finscan)) 
+  fatal("Pick one of -t, -s, and -U.  They all do a TCP portscan.\
+ If you are trying to do TCP SYN scanning, just use -s, for FIN use -U, and \
+ for normal connect() style scanning, use -t");
+if ((o.fragscan && !o.synscan && !o.finscan)) {
+  printf("Specified -f but don't know whether to fragment SYN (-s) scan or FIN (-U) scan.  Doing fragmented SYN scan\n");
+}
+if (o.udpscan || o.lamerscan) 
+  printf("Warning: udp scan is not always 100%c accurate, I will be rewriting it\n", '%'); /* Due to gcc -Wall stupidity */
 if ((o.synscan || o.finscan || o.fragscan || pingscan) && !o.isr00t)
   fatal("Options specified require r00t privileges.  You don't have them!");
-if (!tcpscan && !udpscan && !o.synscan && !o.finscan && !bouncescan && !pingscan) {
+if (!tcpscan && !o.udpscan && !o.synscan && !o.finscan && !bouncescan && !pingscan) {
   tcpscan++;
   if (o.verbose) error("No scantype specified, assuming vanilla tcp connect()\
- scan. Use -P if you really don't want to portscan.");
+ scan. Use -P if you really don't want to portscan (and just want to ping).");
 if (fastscan && ports)
   fatal("You can use -F (fastscan) OR -p for explicit port specification.\
   Not both!\n");
@@ -121,9 +141,6 @@ if (o.max_sockets > MAX_SOCKETS_ALLOWED) {
    printf("Warning: You are limited to MAX_SOCKETS_ALLOWD (%d) paralell sockets.  If you really need more, change the #define and recompile.\n", MAX_SOCKETS_ALLOWED);
    o.max_sockets = MAX_SOCKETS_ALLOWED;
 }
-if (pingscan && o.dontping) {
-  fatal("Ummm ...\"pingscan\" and don't-ping (-D) are INCOMPATIBLE options.  Sorry.\n");
-}
 
 /* If he wants to bounce off of an ftp site, that site better damn well be reachable! */
 if (bouncescan) {
@@ -139,6 +156,15 @@ if (bouncescan) {
     printf("Resolved ftp bounce attack proxy to %s (%s).\n", 
 	   ftp.server_name, inet_ntoa(ftp.server)); 
 }
+fflush(stdout);
+
+if (o.logfd) {
+  /* Brief info incase they forget what was scanned */
+  fprintf(o.logfd, "# Log of: ");
+  for(i=0; i < argc; i++)
+    fprintf(o.logfd, "%s ", fakeargv[i]);
+  fprintf(o.logfd, "\n");
+}
 printf("\nStarting nmap V. %s by Fyodor (fyodor@dhp.com, www.dhp.com/~fyodor/nmap/)\n", VERSION);
 /* I seriously doubt anyone likes this "feature"
 if (!o.verbose) 
@@ -146,7 +172,7 @@ if (!o.verbose)
   */
 if (fastscan)
   ports = getfastports(o.synscan|tcpscan|o.fragscan|o.finscan|bouncescan,
-                       udpscan|lamerscan);
+                       o.udpscan|o.lamerscan);
 if (!ports && !pingscan) ports = getpts("1-1024");
 
 
@@ -192,16 +218,23 @@ while(optind < argc) {
 if (!pingscan) {
   if (!o.dontping && (currenths->flags & HOST_UP) && (o.verbose || o.debugging)) 
     printf("Host %s (%s) appears to be up ... good.\n", currenths->name, inet_ntoa(currenths->host));    
-  else if (o.verbose && !o.dontping && !(currenths->flags & HOST_UP)) 
-    printf("Host %s (%s) appears to be down, skipping it.\n", currenths->name, inet_ntoa(currenths->host));
+  else if (o.verbose && !o.dontping && !(currenths->flags & HOST_UP)) {  
+    if (resolve_all)
+      nmap_log("Host %s (%s) appears to be down, skipping it.\n", currenths->name, inet_ntoa(currenths->host));
+    else printf("Host %s (%s) appears to be down, skipping it.\n", currenths->name, inet_ntoa(currenths->host));
+  }
 }
 else {
   if (currenths->flags & HOST_UP) 
-    printf("Host %s (%s) appears to be up.\n", currenths->name, inet_ntoa(currenths->host));    
+    nmap_log("Host %s (%s) appears to be up.\n", currenths->name, inet_ntoa(currenths->host));    
   else 
-    if (o.verbose || o.debugging || resolve_all) printf("Host %s (%s) appears to be down.\n", currenths->name, inet_ntoa(currenths->host));
+    if (o.verbose || o.debugging || resolve_all) {    
+      if (resolve_all)
+	nmap_log("Host %s (%s) appears to be down.\n", currenths->name, inet_ntoa(currenths->host));
+      else printf("Host %s (%s) appears to be down.\n", currenths->name, inet_ntoa(currenths->host));
+    }
   if (currenths->wierd_responses)
-    printf("Host  %s (%s) seems to be a subnet broadcast address (returned %d extra pings)\n",  currenths->name, inet_ntoa(currenths->host), currenths->wierd_responses);
+    nmap_log("Host  %s (%s) seems to be a subnet broadcast address (returned %d extra pings)\n",  currenths->name, inet_ntoa(currenths->host), currenths->wierd_responses);
 }
 if (currenths->flags & HOST_UP && !currenths->source_ip.s_addr && ( o.synscan || o.finscan)) {
   if (gethostname(myname, MAXHOSTNAMELEN) || 
@@ -229,25 +262,25 @@ if (currenths->flags & HOST_UP && (o.finscan || o.synscan) && !ipaddr2devname( c
 	if (ftp.sd <= 0) ftp_anon_connect(&ftp);
 	if (ftp.sd > 0) bounce_scan(currenths, ports, &ftp);
 	  }
-      if (udpscan) {
-	if (!o.isr00t || lamerscan) 
+      if (o.udpscan) {
+	if (!o.isr00t || o.lamerscan) 
 	  lamer_udp_scan(currenths, ports);
 
 	else udp_scan(currenths, ports);
       }
     
       if (!currenths->ports && !pingscan) {
-	printf("No ports open for host %s (%s)\n", currenths->name,
+	nmap_log("No ports open for host %s (%s)\n", currenths->name,
 	       inet_ntoa(currenths->host));
 	if (currenths->wierd_responses)
-	  printf("Host  %s (%s) seems to be a subnet broadcast address (returned %d extra pings)\n",  currenths->name, inet_ntoa(currenths->host), currenths->wierd_responses);
+	  nmap_log("Host  %s (%s) seems to be a subnet broadcast address (returned %d extra pings)\n",  currenths->name, inet_ntoa(currenths->host), currenths->wierd_responses);
       }
       if (currenths->ports) {
-	printf("Open ports on %s (%s):\n", currenths->name, 
+	nmap_log("Open ports on %s (%s):\n", currenths->name, 
 	       inet_ntoa(currenths->host));
 	printandfreeports(currenths->ports);
 	if (currenths->wierd_responses)
-	  printf("Host  %s (%s) seems to be a subnet broadcast address (returned %d extra pings)\n",  currenths->name, inet_ntoa(currenths->host), currenths->wierd_responses);
+	  nmap_log("Host  %s (%s) seems to be a subnet broadcast address (returned %d extra pings)\n",  currenths->name, inet_ntoa(currenths->host), currenths->wierd_responses);
       }
     }
     fflush(stdout);
@@ -421,37 +454,39 @@ return realloc(ports, portindex * sizeof(unsigned short));
 }
 
 void printusage(char *name) {
-printf("%s [options] [hostname[/mask] . . .]\n\
+printf("nmap V. %s usage: %s [options] [hostname[/mask] . . .]\n\
 options (none are required, most can be combined):\n\
    -t tcp connect() port scan\n\
    -s tcp SYN stealth port scan (must be root)\n\
-   -u UDP port scan, will use MUCH better version if you are root\n\
    -U Uriel Maimon (P49-15) style FIN stealth scan.\n\
-   -l Do the lamer UDP scan even if root.  Less accurate.\n\
    -P ping \"scan\". Find which hosts on specified network(s) are up.\n\
-   -D Don't ping hosts (needed to scan www.microsoft.com and others)\n\
    -b <ftp_relay_host> ftp \"bounce attack\" port scan\n\
+   -u UDP port scan, will use MUCH better version if you are root\n\
+   -l Do the lamer UDP scan even if root.  Less accurate.\n\
    -f use tiny fragmented packets for SYN or FIN scan.\n\
+   -D Don't ping hosts (needed to scan www.microsoft.com and others)\n\
    -i Get identd (rfc 1413) info on listening TCP processes.\n\
-   -n Don't DNS resolve anything unless we have too (makes ping scans faster)\n\
    -p <range> ports: ex: \'-p 23\' will only try port 23 of the host(s)\n\
                   \'-p 20-30,63000-\' scans 20-30 and 63000-65535 default: 1-1024\n\
    -F fast scan. Only scans ports in /etc/services, a la strobe(1).\n\
+   -n Don't DNS resolve anything unless we have to (makes ping scans faster)\n\
    -L <num> Number of pings to perform in parallel.  Your default is: %d\n\
+   -o <logfile> Output scan logs to <logfile>.\n\
    -R Try to resolve all hosts, even down ones (can take a lot of time)\n\
-   -r randomize target port scanning order.\n\
-   -h help, print this junk.  Also see http://www.dhp.com/~fyodor/nmap/\n\
-   -S If you want to specify the source address of SYN or FYN scan.\n", name, LOOKAHEAD);
+   -r do NOT randomize target port scanning order.\n\
+   -S If you want to specify the source address of SYN or FYN scan.\n", VERSION, name, LOOKAHEAD);
+
+
 if (!o.allowall) printf("-A Allow scanning .0 and .255 addresses" );
 printf("   -T <seconds> Set the ping and tcp connect() timeout.\n\
-   -V Print version number and exit.\n\
    -v Verbose.  Its use is recommended.  Use twice for greater effect.\n\
+   -h help, print this junk.  Also see http://www.dhp.com/~fyodor/nmap/\n\
+   -V Print version number and exit.\n\
    -w <n> delay.  n microsecond delay. Not recommended unless needed.\n\
-   -M <n> maximum number of parallel sockets.  Larger isn't always better.\n\
-   -q quash argv to something benign, currently set to \"%s\".\n\
-Hostnames specified as internet hostname or IP address.  Optional '/mask' \
-specifies subnet. cert.org/24 or 192.88.209.5/24 scan CERT's Class C.\n",
-        FAKE_ARGV);
+   -M <n> maximum number of parallel sockets.  Larger isn't always better.\n");
+printf("   -q quash argv to something benign, currently set to \"%s\". (deprecated)\n", FAKE_ARGV);
+printf("Hostnames specified as internet hostname or IP address.  Optional '/mask' \
+specifies subnet. cert.org/24 or 192.88.209.5/24 or 192.88.209.0-255 scan CERT's Class C.\n");
 exit(0);
 }
 
@@ -477,6 +512,7 @@ int timeouts=0;
  static int threshold_warning = 0; /* Have we given the threshold warning yet?*/
 signal(SIGPIPE, SIG_IGN); /* ignore SIGPIPE so our 'write 0 bytes' test
 			     doesn't crash our program!*/
+
 owner[0] = '\0';
 starttime = time(NULL);
 bzero((char *)&sock,sizeof(struct sockaddr_in));
@@ -675,9 +711,9 @@ while(portarray[j] || retryindex >= 0 || current_out != 0) {
 	      timeouts++;	      
 	      if (timeouts > MAX_TIMEOUTS && !target->ports && !o.force) {
 		if (!o.verbose && !o.debugging  && !threshold_warning)
-		  printf("MAX_TIMEOUT threshold (%d) reached, giving up on host %s (%s).  Use -N to skip this check.  This warning won't be repeated during this session\n", MAX_TIMEOUTS, target->name, inet_ntoa(target->host));
+		  nmap_log("MAX_TIMEOUT threshold (%d) reached, giving up on host %s (%s).  Use -N to skip this check.  This warning won't be repeated during this session\n", MAX_TIMEOUTS, target->name, inet_ntoa(target->host));
 		else if (o.verbose || o.debugging)
-		  printf("MAX_TIMEOUT threshold (%d) reached, giving up on host %s (%s).  Use -N to skip this check.\n", MAX_TIMEOUTS, target->name, inet_ntoa(target->host));
+		  nmap_log("MAX_TIMEOUT threshold (%d) reached, giving up on host %s (%s).  Use -N to skip this check.\n", MAX_TIMEOUTS, target->name, inet_ntoa(target->host));
 		threshold_warning++;
 		 for(k=0; k < o.max_sockets; k++) 
 		   if (portno[k]) 
@@ -718,9 +754,9 @@ while(portarray[j] || retryindex >= 0 || current_out != 0) {
 	  timeouts++;	      
 	  if (timeouts > MAX_TIMEOUTS && !target->ports && !o.force) {
 	    if (!o.verbose && !o.debugging  && !threshold_warning)
-	      printf("MAX_TIMEOUT threshold (%d) reached, giving up on host %s (%s).  Use -N to skip this check.  This warning won't be repeated during this session\n", MAX_TIMEOUTS, target->name, inet_ntoa(target->host));
+	      nmap_log("MAX_TIMEOUT threshold (%d) reached, giving up on host %s (%s).  Use -N to skip this check.  This warning won't be repeated during this session\n", MAX_TIMEOUTS, target->name, inet_ntoa(target->host));
 	    else if (o.verbose || o.debugging)
-	      printf("MAX_TIMEOUT threshold (%d) reached, giving up on host %s (%s).  Use -N to skip this check.\n", MAX_TIMEOUTS, target->name, inet_ntoa(target->host));
+	      nmap_log("MAX_TIMEOUT threshold (%d) reached, giving up on host %s (%s).  Use -N to skip this check.\n", MAX_TIMEOUTS, target->name, inet_ntoa(target->host));
 	    threshold_warning++;
 	    for(k=0; k < o.max_sockets; k++) 
 	      if (portno[k]) 
@@ -857,12 +893,12 @@ void printandfreeports(portlist ports) {
   struct servent *service;
   port *current = ports, *tmp;
   
-  printf("Port Number  Protocol  Service");
-  printf("%s", (o.identscan)?"         Owner\n":"\n");
+  nmap_log("Port Number  Protocol  Service");
+  nmap_log("%s", (o.identscan)?"         Owner\n":"\n");
   while(current != NULL) {
     strcpy(protocol,(current->proto == IPPROTO_TCP)? "tcp": "udp");
     service = getservbyport(htons(current->portno), protocol);
-    printf("%-13d%-11s%-16s%s\n", current->portno, protocol,
+    nmap_log("%-13d%-11s%-16s%s\n", current->portno, protocol,
 	   (service)? service->s_name: "unknown",
 	   (current->owner)? current->owner : "");
     tmp = current;
@@ -870,7 +906,7 @@ void printandfreeports(portlist ports) {
     if (tmp->owner) free(tmp->owner);
     free(tmp);
   }
-  printf("\n");
+  nmap_log("\n");
 }
 
 /* This is the version of udp_scan that uses raw ICMP sockets and requires 
@@ -1397,7 +1433,7 @@ if (!target->source_ip.s_addr) {
  * apparently requires that for some unknown reason(!).  I'd like to
  * get out of promisc. if we can figure out the problem.
  */
-if (!(pd = pcap_open_live(target->device, 92, 0, 1500, err0r)))
+if (!(pd = pcap_open_live(target->device, 92, (o.spoofsource)? 1 : 0, 1500, err0r)))
   fatal("pcap_open_live: %s", err0r);
 if (pcap_lookupnet(target->device, &localnet, &netmask, err0r) < 0)
   fatal("Failed to lookup device subnet/netmask: %s", err0r);
@@ -1564,9 +1600,9 @@ ip->ip_v = 4;
 ip->ip_hl = 5;
 /*RFC 791 allows 8 octet frags, but I get "operation not permitted" (EPERM)
   when I try that.  */
-ip->ip_len = htons(sizeof(struct ip) + 16);
+ip->ip_len = BSDFIX(sizeof(struct ip) + 16);
 id = ip->ip_id = rand();
-ip->ip_off = htons(MORE_FRAGMENTS);
+ip->ip_off = BSDFIX(MORE_FRAGMENTS);
 ip->ip_ttl = 255;
 ip->ip_p = IPPROTO_TCP;
 ip->ip_src.s_addr = source->s_addr;
@@ -1585,7 +1621,7 @@ if (o.debugging > 1)
 /* Lets save this and send it AFTER we send the second one, just to be
    cute ;) */
 
-if ((res = sendto(sd, packet, ntohs(ip->ip_len), 0, 
+if ((res = sendto(sd, packet,sizeof(struct ip) + 16 , 0, 
 		  (struct sockaddr *)&sock, sizeof(struct sockaddr_in))) == -1)
   {
     perror("sendto in send_syn_fragz");
@@ -1598,9 +1634,9 @@ if (o.debugging > 1) printf("successfully sent %d bytes of raw_tcp!\n", res);
 bzero((char *) ip2, sizeof(struct ip));
 ip2->ip_v= 4;
 ip2->ip_hl = 5;
-ip2->ip_len = htons(sizeof(struct ip) + 4); /* the rest of our TCP packet */
+ip2->ip_len = BSDFIX(sizeof(struct ip) + 4); /* the rest of our TCP packet */
 ip2->ip_id = id;
-ip2->ip_off = htons(2);
+ip2->ip_off = BSDFIX(2);
 ip2->ip_ttl = 255;
 ip2->ip_p = IPPROTO_TCP;
 ip2->ip_src.s_addr = source->s_addr;
@@ -1616,9 +1652,8 @@ if (o.debugging > 1)
 
   printf("\nTrying sendto(%d , ip2, %d, 0 , %s , %d)\n", sd, 
 	 ntohs(ip2->ip_len), inet_ntoa(*victim), (int) sizeof(struct sockaddr_in));
-if ((res = sendto(sd, (void *)ip2, ntohs(ip2->ip_len), 0, 
+if ((res = sendto(sd, (void *)ip2,sizeof(struct ip) + 4 , 0, 
 		  (struct sockaddr *)&sock, (int) sizeof(struct sockaddr_in))) == -1)
-
   {
     perror("sendto in send_tcp_raw frag #2");
     return -1;
@@ -1679,7 +1714,7 @@ if (!target->source_ip.s_addr) {
  * get out of promisc. if we can figure out the problem.
  */
 
-if (!(pd = pcap_open_live(target->device, 92, 0, 1500, err0r)))
+if (!(pd = pcap_open_live(target->device, 92,  (o.spoofsource)? 1 : 0, 1500, err0r)))
   fatal("pcap_open_live: %s", err0r);
 
 if (pcap_lookupnet(target->device, &localnet, &netmask, err0r) < 0)
@@ -1927,11 +1962,12 @@ int retriesleft = FTP_RETRIES;
 char recvbuf[2048]; 
 char targetstr[20];
 char command[512];
+unsigned short portno,p1,p2;
 
 #ifndef HAVE_SNPRINTF
-sprintf(targetstr, "%d,%d,%d,%d,0,", UC(t[0]), UC(t[1]), UC(t[2]), UC(t[3]));
+sprintf(targetstr, "%d,%d,%d,%d,", UC(t[0]), UC(t[1]), UC(t[2]), UC(t[3]));
 #else
-  snprintf(targetstr, 20, "%d,%d,%d,%d,0,", UC(t[0]), UC(t[1]), UC(t[2]), UC(t[3]));
+  snprintf(targetstr, 20, "%d,%d,%d,%d,", UC(t[0]), UC(t[1]), UC(t[2]), UC(t[3]));
 #endif
 
 starttime = time(NULL);
@@ -1939,11 +1975,15 @@ if (o.verbose || o.debugging)
   printf("Initiating TCP ftp bounce scan against %s (%s)\n",
 	 target->name,  inet_ntoa(target->host));
 for(i=0; portarray[i]; i++) {
+  portno = htons(portarray[i]);
+  p1 = ((char *) &portno)[0];
+  p2 = ((char *) &portno)[1];
 #ifndef HAVE_SNPRINTF
-  sprintf(command, "PORT %s%i\r\n", targetstr, portarray[i]);
+  sprintf(command, "PORT %s%i,%i\r\n", targetstr, p1,p2);
 #else
-  snprintf(command, 512, "PORT %s%i\r\n", targetstr, portarray[i]);
+  snprintf(command, 512, "PORT %s%i,%i\r\n", targetstr, p1,p2);
 #endif
+  if (o.debugging) printf("Attempting command: %s", command);
   if (send(sd, command, strlen(command), 0) < 0 ) {
     perror("send in bounce_scan");
     if (retriesleft) {
@@ -2069,3 +2109,42 @@ ftp->user[63] = ftp->pass[255] = ftp->server_name[MAXHOSTNAMELEN] = 0;
 return 1;
 }
 
+void nmap_log(char *fmt, ...) {
+va_list  ap;
+va_start(ap, fmt);
+vfprintf(stdout, fmt, ap);
+fflush(stdout);
+if (o.logfd && o.logfd != stdout) {
+  vfprintf(o.logfd, fmt, ap);
+}
+va_end(ap);
+return;
+}
+
+void sigdie(int signo) {
+ switch(signo) {
+ case SIGINT:
+   fprintf(stderr, "caught SIGINT signal, cleaning up\n");
+   break;
+ case SIGTERM:
+   fprintf(stderr, "caught SIGTERM signal, cleaning up\n");
+   break;
+ case SIGHUP:
+   fprintf(stderr, "caught SIGHUP signal, cleaning up\n");
+   break;
+ case SIGSEGV:
+   fprintf(stderr, "caught SIGSEGV signal, cleaning up\n");
+   break;
+ case SIGBUS:
+   fprintf(stderr, "caught SIGBUS signal, cleaning up\n");
+   break;
+ default:
+   fprintf(stderr, "caught signal %d, cleaning up\n", signo);
+   break;
+ }
+ fflush(stdout);
+ if (o.logfd && o.logfd != stdout) fclose(o.logfd);
+ exit(1);
+}
+
+
